@@ -94,10 +94,18 @@ ipmi_log(enum ipmi_log_type_e log_type, const char *format, ...)
     va_list ap;
 
     va_start(ap, format);
-    if (ipmi_os_handler->vlog)
+    if (ipmi_os_handler && ipmi_os_handler->vlog)
 	ipmi_os_handler->vlog(ipmi_os_handler, log_type, format, ap);
-    else
+    else {
 	vfprintf(stderr, format, ap);
+	switch (log_type) {
+	case IPMI_LOG_DEBUG_START:
+	case IPMI_LOG_DEBUG_CONT:
+	    break;
+	default:
+	    fprintf(stderr, "\n");
+	}
+    }
     va_end(ap);
 }
 
@@ -919,6 +927,10 @@ void _ipmi_normal_fru_shutdown(void);
 void _ipmi_fru_spd_decoder_shutdown(void);
 int _ipmi_sol_init(void);
 
+void _ipmi_rakp_shutdown(void);
+void _ipmi_aes_cbc_shutdown(void);
+void _ipmi_hmac_shutdown(void);
+void _ipmi_md5_shutdown(void);
 void ipmi_oem_atca_conn_shutdown(void);
 void ipmi_oem_intel_shutdown(void);
 void ipmi_oem_kontron_conn_shutdown(void);
@@ -1027,6 +1039,10 @@ ipmi_shutdown(void)
     if (! ipmi_initialized)
 	return;
 
+    _ipmi_rakp_shutdown();
+    _ipmi_aes_cbc_shutdown();
+    _ipmi_hmac_shutdown();
+    _ipmi_md5_shutdown();
     _ipmi_sol_shutdown();
     _ipmi_lan_shutdown();
 #ifdef HAVE_OPENIPMI_SMI
